/* BdfServer_DirectoryStorage - Copyright (c) 2009-2017 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.storage.directory.implementations;

import fr.exemole.bdfserver.api.storage.BdfExtensionStorage;
import fr.exemole.bdfserver.api.storage.BdfStorageException;
import java.io.File;


/**
 *
 * @author Vincent Calame
 */
public class BdfExtensionStorageImpl implements BdfExtensionStorage {

    private final File varDirectory;
    private final File backupDirectory;

    public BdfExtensionStorageImpl(File dataDirectory, File backupDirectory) {
        this.varDirectory = dataDirectory;
        this.backupDirectory = backupDirectory;
    }

    @Override
    public File getDataDirectory(String extensionName) {
        File dir = new File(varDirectory, extensionName);
        if ((dir.exists()) && (!dir.isDirectory())) {
            throw new BdfStorageException(dir.getAbsolutePath() + " is not a directory");
        }
        return dir;
    }

    @Override
    public File getBackupDirectory(String extensionName) {
        if (backupDirectory == null) {
            return null;
        }
        File dir = new File(backupDirectory, extensionName);
        if ((dir.exists()) && (!dir.isDirectory())) {
            throw new BdfStorageException(dir.getAbsolutePath() + " is not a directory");
        }
        return dir;
    }

}
